﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Library
{
    public struct Vector2
    {
        public int X, Y;
        public Vector2(Point v) { X = v.X; Y = v.Y; }
        public Vector2(Size v) { X = v.Width; Y = v.Height; }
        public Vector2(int x, int y) { X = x; Y = y; }

        public static Vector2 operator +(Vector2 a, Vector2 b) { return new Vector2(a.X + b.X, a.Y + b.Y); }
        public static Vector2 operator -(Vector2 a) { return new Vector2(-a.X, -a.Y); }
        public static Vector2 operator -(Vector2 a, Vector2 b) { return new Vector2(a.X - b.X, a.Y - b.Y); }
        public static Vector2 operator *(int a, Vector2 b) { return new Vector2(a * b.X, a * b.Y); }
        public static Vector2 operator *(Vector2 a, int b) { return new Vector2(a.X * b, a.Y * b); }
        public static Vector2 operator /(Vector2 a, int b) { return new Vector2(a.X / b, a.Y / b); }
        public static bool operator ==(Vector2 a, Vector2 b) { return a.X == a.Y && b.X == b.Y; }
        public static bool operator !=(Vector2 a, Vector2 b) { return !(a == b); }
        public override int GetHashCode() { return X << 16 | Y; }
        public override bool Equals(object obj) { return obj is Vector2 ? this == (Vector2)obj : false; }

        public static implicit operator System.Drawing.Point(Vector2 v) { return new System.Drawing.Point((int)v.X, (int)v.Y); }
        public static implicit operator System.Drawing.Size(Vector2 v) { return new System.Drawing.Size((int)v.X, (int)v.Y); }
        public static explicit operator Vector2(System.Drawing.Point v) { return new Vector2(v); }
        public static explicit operator Vector2(System.Drawing.Size v) { return new Vector2(v); }

        //public int Length { get { return Math.Sqrt(this.X * this.X + this.Y * this.Y); } }
        //public Vector2 Norm() { return new Vector2(this.X / Length, this.Y / Length); }
        //public int Dot(Vector2 with) { return this.X * with.X + this.Y * with.Y; }
        //public int Cross(Vector2 with) { return this.X * with.Y - this.Y * with.X; }

    }

    public struct Vector3
    {
        public int X, Y, Z;
        public Vector3(int x, int y, int z) { X = x; Y = y; Z = z; }

        public static Vector3 operator +(Vector3 a, Vector3 b) { return new Vector3(a.X + b.X, a.Y + b.Y, a.Z + b.Z); }
        public static Vector3 operator -(Vector3 a, Vector3 b) { return new Vector3(a.X - b.X, a.Y - b.Y, a.Z - b.Z); }
        public static Vector3 operator *(int a, Vector3 b) { return new Vector3(a * b.X, a * b.Y, a * b.Z); }
        public static Vector3 operator *(Vector3 a, int b) { return new Vector3(a.X * b, a.Y * b, a.Z * b); }
        public static Vector3 operator /(Vector3 a, int b) { return new Vector3(a.X / b, a.Y / b, a.Z / b); }

        //public int Length { get { return Math.Sqrt(this.X * this.X + this.Y * this.Y + this.Z * this.Z); } }
        //public Vector3 Norm() { return new Vector3(this.X / Length, this.Y / Length, this.Z / Length); }
        //public int Dot(Vector3 with) { return this.X * with.X + this.Y * with.Y + this.Z * with.Z; }
        //public Vector3 Cross(Vector3 with) { return new Vector3(this.Y * with.Z - this.Z * with.Y, this.Z * with.X - this.X - with.Z, this.X * with.Y - this.Y * with.X); }
    }

    public struct Rect
    {
        Vector2 pt, pt2, sz;
        public Rect(Point point, Size size) { pt = (Vector2)point; sz = (Vector2)size; pt2 = pt + sz; }
        public Rect(int x, int y, int w, int h) { pt = new Vector2(x, y); sz = new Vector2(w, h); pt2 = pt + sz; }

        public int X { get { return pt.X; } set { pt.X = value; } }
        public int Y { get { return pt.Y; } set { pt.Y = value; } }
        public int W { get { return sz.X; } set { sz.X = value; } }
        public int H { get { return sz.Y; } set { sz.Y = value; } }
        public int X2 { get { return pt2.X; } }
        public int Y2 { get { return pt2.Y; } }

        public Vector2 Point { get { return pt; } set { pt = value; pt2 = pt + sz; } }
        public Vector2 Size { get { return sz; } set { sz = value; pt2 = pt + sz; } }
        public Vector2 Point2 { get { return pt2; } }

        public Rect Offset(Vector2 point) { return new Rect(pt + point, sz); }
        public Rect Offset(int x, int y) { return new Rect(pt + new Vector2(x, y), sz); }
        public Rect Inflate(Vector2 size) { return new Rect(pt, sz + size); }
        public Rect Inflate(int w, int h) { return new Rect(pt, sz + new Vector2(w, h)); }
        
        public bool Contains(Vector2 point) { return pt.X <= point.X && pt.Y <= point.Y && pt2.X >= point.X && pt2.Y >= point.Y; }
        public bool IntersectWith(Rect rect) { return X <= rect.X2 && Y <= rect.Y2 && rect.X <= X2 && rect.Y <= Y2; }

        public static implicit operator System.Drawing.Rectangle(Rect r)
        {
            return new System.Drawing.Rectangle(r.pt, r.sz);
        }
    }
}
