﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

using Tsukikage.GameSDK.GSystem;
using Tsukikage.GameSDK.GSystem.Settings;

namespace Kuzushi
{
    class Program
    {
        /// <summary>
        /// エントリポイントを記述します。
        /// </summary>
        /// <param name="args"></param>
        [STAThread]
        public static void Main(string[] args)
        {
            // まずはゲームに関する諸設定
            GSDKConfig config = new GSDKConfig();

            // ゲーム名とか
            config.GameTitle = "くずし";
            config.GameVersion = "v1.00";
            config.GameAuthor = "つーさ";

            // 画面サイズとか
            config.ScreenSize = new Size(640, 480);
            config.FPS = 60.0;

            // 最初の画面
            config.StartScene = new Scenes.Start();


            #region キーコンフィグとか

            // ActionsのOffset + TypesのOffsetが実際のボタン番号に割り当てられる。
            // 以下の例で 2PのAction1 は 4+16 = 20番が実際のボタン番号

            // 行項目
            config.KeyConfig.Actions = new KCEntry[]{
                new KCEntry("Up", 0),
                new KCEntry("Down", 1),
                new KCEntry("Left", 2),
                new KCEntry("Right", 3),
                null,
                new KCEntry("Action1", 4),
                new KCEntry("Action2", 5),
                new KCEntry("Action3", 6),
                new KCEntry("Action4", 7),
            };

            // 列項目
            config.KeyConfig.Types = new KCEntry[]{
                new KCEntry("1P", 0),
                new KCEntry("1P(Sub)", 0),
                new KCEntry("2P", 16),
                new KCEntry("2P(Sub)", 16),
            };

            // これは、DebugモードでConfigureを開いて、適当に設定してからAsDefaultボタンをぽちっと押すと得られる
            config.KeyConfig.Entries = new int[] { 200, 16842960, 33685707, 50528461, 84148268, 100991021, 117833774, 134676527, 268435712, 285278465, 302121218, 318963971, 352583940, 369426693, 386269446, 403112199, };
            #endregion

            // コマンドラインに /config が含まれていたら
            if (Environment.CommandLine.IndexOf("/config", StringComparison.CurrentCultureIgnoreCase) != -1)
            {
                // 設定ダイアログを起動。
                if (GSDK.Configure(config) != DialogResult.Yes)
                {
                    // Yes 以外ならゲームを起動する必要はない
                    return;
                }
            }

            // configに則してゲーム開始
            GSDK.Run(config);
        }
    }
}
